<?php

// TODO Set our own cache bin.
define('VIEWS_NAVIGATION_CACHE_BIN', 'cache');

/**
 * Get a view query from cache.
 */
function views_navigation_get_cached_query($cid) {
  $cache = cache_get('query-' . $cid, VIEWS_NAVIGATION_CACHE_BIN);
  // ATM we only handle the default views query backend.
  if ($cache && is_a($cache->data, 'views_plugin_query_default')) {
    return $cache->data;
  }
}

/**
 * Get a view result from cache.
 */
function views_navigation_get_cached_result($cid) {
  $cache = cache_get('result-' . $cid, VIEWS_NAVIGATION_CACHE_BIN);
  return $cache ? $cache->data : FALSE;
}

/**
 * Store a view query in cache.
 * Return the cid or FALSE if the query is not stored.
 */
function views_navigation_store_query($view) {
  // If there is zero or one result, we do nothing.
  if ($view->total_rows < 2) {
    return FALSE;
  }
  // ATM we only handle the default views query backend.
  if (!is_a($view->query, 'views_plugin_query_default')) {
    return FALSE;
  }

  // Remove useless properties. This is the not easy part.
  $query_to_store = clone($view->query);
  unset($query_to_store->display);
  unset($query_to_store->header);
  unset($query_to_store->limit);
  unset($query_to_store->pager);
  $view_to_store = clone($query_to_store->view);
  foreach ($view_to_store as $key => $value) {
    if (!in_array($key, array(
      'base_database',
      'name'
    ))) {
      unset($view_to_store->$key);
    }
  }
  $query_to_store->view = $view_to_store;

  $cid = views_navigation_get_query_cid($query_to_store);
  // The query may be stored already.
  if (!views_navigation_get_cached_query($cid)) {
    cache_set('query-' . $cid, $query_to_store, VIEWS_NAVIGATION_CACHE_BIN);
  }
  return $cid;
}

/**
 * Get the unique cid corresponding to a view query.
 */
function views_navigation_get_query_cid($query) {
  return drupal_hash_base64(serialize($query));
}

/**
 * Redirect to the next or previous node.
 */
function views_navigation_router($cid, $pos) {
  if ($result = views_navigation_get_result($cid)) {
    // Manage array ends, cycling behavior.
    $max_index = count($result) - 1;
    $pos = ($pos > $max_index) ? 0 : ($pos < 0) ? $max_index : $pos;

    if ($nid = $result[$pos]) {
      $params = array(
        'vnc' => $cid,
        'vnp' => $pos,
      );
      drupal_goto('node/' . $nid, array('query' => $params));
    }
  }
  // Something went wrong.
  watchdog('views_navigation', t('Could not find any next/previous item.'), func_get_args(), WATCHDOG_ERROR);
  return t("Sorry, could not find any next/previous item.");
}

/**
 * Get the result of a query, as an array of nids keyed by position.
 */
function views_navigation_get_result($cid) {
  if (!$result = views_navigation_get_cached_result($cid)) {
    if ($query = views_navigation_get_cached_query($cid)) {
      $real_query = $query->query();
      $rows = $real_query->execute();
      $result = array();
      foreach ($rows as $pos => $row) {
        $result[$pos] = $row->nid;
      }
    }
  }
  return $result;
}
